#include "inkey.ch"
#include "setcurs.ch"
#include "box.ch"

FUNCTION Test()
	LOCAL cText

	SET SCOREBOARD OFF

	cText := WordPerf(0, 0, MAXROW(), MAXCOL(), "W/B")
	CLEAR SCREEN

	RETURN .T.


FUNCTION WordPerf(nTop, nLeft, nBottom, nRight, cColor, cFile)
	LOCAL oEdit
	LOCAL nKey := 0
	LOCAL lQuit := .F.
	LOCAL cSaveColor
	LOCAL cSaveScr
	LOCAL nHome := 0
	LOCAL i
	LOCAL nRepeat := 1
	LOCAL lDone := .F.
	LOCAL cBlock := ""
	LOCAL nChoice

	// draw screen
	cSaveScr := SAVESCREEN(nTop, nLeft, nBottom, nRight)
	cSaveColor := SETCOLOR(cColor)
	SCROLL(nTop, nLeft, nBottom, nRight, 0)


	oEdit := NewEdit(nTop, nLeft, nBottom-1, nRight, cColor, cFile)
	oEdit:insert := .T.

	DO WHILE !lQuit

		DISPBEGIN()	

		SETCURSOR(SC_NONE)
		@ nBottom, 0
		@ nBottom, MAXCOL()-15 SAY "Line "+LTRIM(STR(oEdit:nLine))+ ;
		  "  Pos "+LTRIM(STR(oEdit:nCol))

		// stabilize current edit window -- NOT incremental!
		oEdit:stabilize()

		SETCURSOR(IF(oEdit:insert, SC_INSERT, SC_NORMAL))
		DISPEND()

		// wait for keystroke
		nKey := INKEY(0)

		nHome := IF(nHome == 3, 0, nHome)
		DO CASE
		CASE nHome == 1
			nHome := 3
			DO CASE
			CASE nKey == K_HOME
				nHome := 2
			CASE nKey == K_DOWN
				oEdit:down(oEdit:nBottom - oEdit:nTop - oEdit:nRow)
			CASE nKey == K_UP
				oEdit:up(oEdit:nRow)
			CASE nKey == K_LEFT
				oEdit:home()
			CASE nKey == K_RIGHT
				oEdit:end()
			OTHERWISE
				nHome := 0
			ENDCASE

		CASE nHome == 2
			nHome := 3
			DO CASE
			CASE nKey == K_DOWN
				oEdit:goBottom()
			CASE nKey == K_UP
				oEdit:goTop()
			OTHERWISE
				nHome := 0
			ENDCASE
		ENDCASE

		IF nHome == 0
			IF nKey == K_ESC
				nRepeat := MsgNum("Repeat Value = ", oEdit:nBottom+1, "99", "8", @nKey)
			ENDIF
			lDone := .F.
			i := 1
			DO WHILE i <= nRepeat .AND. !lDone
				DO CASE
				CASE nKey >= 32 .AND. nKey <= 122
					oEdit:putChar(nKey)
				CASE nKey == K_LEFT
					oEdit:moveLeft()
				CASE nKey == K_RIGHT
					oEdit:moveRight()
				CASE nKey == K_UP
					oEdit:up()
				CASE nKey == K_DOWN
					oEdit:down()
				CASE nKey == K_DEL
					oEdit:delChar()
				CASE nKey == K_PGUP
					oEdit:pageUp()
				CASE nKey == K_PGDN
					oEdit:pageDown()
				CASE nKey == K_F10
					lDone := .T.
					cFile := MsgChar("Document to be saved:", oEdit:nBottom + 1, cFile)
					oEdit:writeText(cFile)
				CASE nKey == K_SH_F10
					lDone := .T.
					cFile := MsgChar("Document to be retrieved:", oEdit:nBottom + 1)
					IF FILE(cFile)
						oEdit := NewEdit(nTop, nLeft, nBottom-1, nRight, cColor)
						oEdit:readText(cFile)
						oEdit:wrap()	// force word-wrap of entire text
					ENDIF
				CASE nKey == K_HOME
					lDone := .T.
					nHome := 1 
				CASE nKey == K_F7
					lDone := .T.
					IF MsgYN("Save document?", oEdit:nBottom + 1, .T.)
						IF EMPTY(cFile)
							cFile := MsgChar("Document to be saved:", oEdit:nBottom + 1)
						ENDIF
						oEdit:writeText(cFile)
					ENDIF
					lQuit := MsgYN("Quit WP?", oEdit:nBottom + 1, .F.)
					IF !lQuit
						oEdit := NewEdit(nTop, nLeft, nBottom-1, nRight, cColor)
					ENDIF

				CASE nKey == 127	// ctrl-backspace
					oEdit:wordLeft()
					oEdit:delWord()
				CASE nKey == K_BS
					oEdit:backspace()
				CASE nKey == K_INS
					lDone := .T.
					oEdit:insert := !oEdit:insert
				CASE nKey == K_END
					oEdit:end()
				CASE nKey == K_HOME
					nHome++
				CASE nKey == K_ALT_D
					oEdit:delLine()
				CASE nKey == K_ALT_I
					oEdit:home()
					oEdit:insLine()
				CASE nKey == K_CTRL_LEFT
					oEdit:wordLeft()
				CASE nKey == K_CTRL_RIGHT
					oEdit:wordRight()
				CASE nKey == K_CTRL_END
					oEdit:delRest()
				CASE nKey == K_ALT_B
					oEdit:putColor(2)
					oEdit:putColor(1)
					oEdit:moveLeft()
				CASE nKey == K_ALT_F4
					IF oEdit:lBlockOn
						oEdit:blockOff()
					ELSE
						oEdit:blockOn()
					ENDIF
				CASE nKey == K_CTRL_F4 .AND. !EMPTY(cBlock)
					nChoice := WPMenu(oEdit:nBottom + 1,{"Move", "Copy", "Delete", "Append"})
					oEdit:lBlockOn := .F.
					DO CASE
					CASE nChoice == 1
						cBlock := oEdit:blockGet()
						oEdit:blockDel()
					CASE nChoice == 2
						cBlock := oEdit:blockGet()
						Paste(oEdit, cBlock)
					CASE nChoice == 3
						oEdit:blockDel()
					CASE nChoice == 4
						cBlock += oEdit:blockGet()
					ENDCASE
					oEdit:blockOff()


				OTHERWISE
					oEdit:putChar(nKey)
				ENDCASE

				i++
			ENDDO
			nRepeat := 1
		ENDIF

	ENDDO


	SETCOLOR(cSaveColor)
	RESTSCREEN(nTop, nLeft, nBottom, nRight, cSaveScr)

	RETURN oEdit:getText()


STATIC FUNCTION Paste(oEdit, cText)
	LOCAL nKey := 0
	DO WHILE nKey != K_ENTER
		// stabilize current edit window -- NOT incremental!
		oEdit:stabilize()

		nKey := INKEY(0)
		IF nKey != K_ENTER
			oEdit:defaultKey(nKey)
		ENDIF
	ENDDO
	oEdit:putChar(cText, .T.)

	RETURN (NIL)


STATIC FUNCTION MsgChar(cMsg, nRow, cDefault)
	LOCAL GetList := {}
	LOCAL cSaveScr

	cDefault := IF(cDefault == NIL, SPACE(35), PADR(cDefault, 35))

	cSaveScr := SAVESCREEN(nRow, 0, nRow, MAXCOL())
	@ nRow, 0 SAY cMsg COLOR "W+/B" GET cDefault PICTURE "@K"
	READ

	RESTSCREEN(nRow, 0, nRow, MAXCOL(), cSaveScr)

	RETURN cDefault


STATIC FUNCTION MsgYN(cMsg, nRow, lResponse)
	LOCAL GetList := {}
	LOCAL cSaveScr

	lResponse := IF(lResponse == NIL, .F., lResponse)

	cSaveScr := SAVESCREEN(nRow, 0, nRow, MAXCOL())
	@ nRow, 0 SAY cMsg COLOR "W+/B" GET lResponse PICTURE "Y"
	READ

	RESTSCREEN(nRow, 0, nRow, MAXCOL(), cSaveScr)

	RETURN lResponse


STATIC FUNCTION WPMenu(nRow, aMenu)
	LOCAL nChoice := 1
	LOCAL GetList := {}
	LOCAL i
	LOCAL nPos := 0
	LOCAL cSaveScr

	cSaveScr := SAVESCREEN(nRow, 0, nRow, MAXCOL())

	FOR i := 1 TO LEN(aMenu)
		@ nRow, nPos SAY STR(i, 1) + " " + LEFT(aMenu[i], 1) COLOR "W+/B"
		@ nRow, nPos+3 SAY SUBSTR(aMenu[i], 2) + IF(i != LEN(aMenu), "; ", ":") COLOR "W/B"
		nPos += LEN(aMenu[i])+4
	NEXT
	@ nRow, nPos GET nChoice PICTURE "9" VALID (nChoice > 0 .AND. nChoice <= LEN(aMenu))
	READ


	RESTSCREEN(nRow, 0, nRow, MAXCOL(), cSaveScr)

	RETURN nChoice


STATIC FUNCTION MsgNum(cMsg, nRow, cPicture, cDefault, nKey)
	LOCAL cSaveScr
	LOCAL cKey
	LOCAL cNum := ""

	cSaveScr := SAVESCREEN(nRow, 0, nRow, MAXCOL())

	@ nRow, 0 SAY cMsg+" "+cDefault COLOR "W+/B"
	SETPOS(nRow, LEN(cMsg)+1)
	nKey := INKEY(0)
	cKey := CHR(nKey)
	DO WHILE (cKey >= '0' .AND. cKey <= '9') .OR. nKey == K_BS
		IF nKey == K_BS
			cNum := IF(cNum == "", "", LEFT(cNum, LEN(cNum)-1))
		ELSE
			cNum += cKey
		ENDIF
		@ nRow, LEN(cMsg)
		@ nRow, LEN(cMsg)+1 SAY cNum
		nKey := INKEY(0)
		cKey := CHR(nKey)
	ENDDO

	RESTSCREEN(nRow, 0, nRow, MAXCOL(), cSaveScr)
	
	IF nKey == K_F1 .OR. nKey == K_F7 .OR. nKey == K_ESC
		cNum := "0"
	ENDIF

	RETURN IF(cNum == "", VAL(cDefault), VAL(cNum))

FUNCTION ColorChg(nCode, nColor)
	DO CASE
	CASE nCode == 1
		RETURN "W/B"
	CASE nCode == 2
		RETURN "W+/B"
	ENDCASE

	RETURN nColor

FUNCTION NewEdit( nTop, nLeft, nBottom, nRight, cColor, cFile )
	LOCAL oEdit

	oEdit := FastEdit():New(nTop, nLeft, nBottom, nRight, cColor)
	IF FILE(cFile)
		oEdit:readText(cFile)
	ENDIF
	oEdit:bColorBlock := {|nCode, nColor| ColorChg(nCode, nColor) }
	oEdit:nTabSize := 5
	oEdit:wordWrap := .T.
	oEdit:wrap()	// force word-wrap of entire text


	RETURN oEdit
